unit ULZMAAlone2;

{$IFDEF FPC}
{$MODE Delphi}
{$ENDIF}

interface

uses ULZMAEncoder,ULZMADecoder,UBufferedFS,ULZMACommon,Classes,dialogs;

const kEncode=0;
      kDecode=1;
      kBenchmark=2;

type TCommandLine=class
       public
         command:integer;
         NumBenchMarkPasses:integer;
         DictionarySize:integer;
         DictionarySizeIsDefined:boolean;
         Lc:integer;
         Lp:integer;
         Pb:integer;

         Fb:integer;
         FbIsDefined:boolean;

         Eos:boolean;

         Algorithm:integer;
         MatchFinder:integer;

         InFile:string;
         OutFile:string;

         constructor Create;
         function ParseSwitch(const s:string):boolean;
         function Parse(inF,OutF:string):boolean;
       end;

     TLZMAAlone=class
       public
         procedure PrintHelp;
         function Main(inF,outF,Cmd:string):boolean;
       end;

implementation

uses SysUtils;

constructor TCommandLine.Create;
begin
Command:=-1;
NumBenchMarkPasses:=10;
DictionarySize:=1 shl 23;
DictionarySizeIsDefined:= false;
Lc:= 3;
Lp:= 0;
Pb:= 2;

Fb:= 128;
FbIsDefined:= false;

Eos:= false;

Algorithm:= 2;
MatchFinder:= 1;
end;

function GetStr(const str:string;const offset:integer):string;
var i:integer;
begin
result:='';
for i:=offset to length(str) do
    result:=result+str[i];
end;

function GetInt(const str:string;const offset:integer):integer;
var s:string;
begin
s:=GetStr(s,offset);
result:=strtoint(s);
end;

function TCommandLine.ParseSwitch(const s:string):boolean;
var l:integer;
    mfs:string;
begin
result:=false;
l:=length(s);
if l=0 then exit;
case s[1] of
     'd': begin
          DictionarySize := 1 shl GetInt(s,2);
          DictionarySizeIsDefined := true;
          result:=true;
          end;
     'f': begin
          if (l>=2)and(s[2]='b') then begin
             fb:=GetInt(s,3);
             FbIsDefined := true;
             result:=true;
             end;
          end;
     'a': begin
          Algorithm := GetInt(s,2);
          result:=true;
          end;
     'l': begin
          if (l>=2) then begin
             if s[2]='c' then begin
                Lc:=GetInt(s,3);
                result:=true;
                end;
             if s[2]='p' then begin
                Lp:=GetInt(s,3);
                result:=true;
                end;
             end;
          end;
     'p': begin
          if (l>=2)and(s[2]='b') then begin
             Pb:=GetInt(s,3);
             result:=true;
             end;
          end;
     'e': begin
          if (l>=3)and(s[2]='o')and(s[3]='s') then begin
             eos:=true;
             result:=true;
             end;
          end;
     'm': begin
          if (l>=2)and(s[2]='f') then begin
             mfs:=GetStr(s,3);
             if mfs='bt2' then
                MatchFinder:=0
             else if mfs='bt4' then
                  MatchFinder:=1
             else if mfs='bt4b' then
                  MatchFinder:=2
             else begin
                  result:=false;
                  exit;
                  end;
             end;
          end;
      else
        result:=false;
      end;
end;

function TCommandLine.Parse(inF,OutF:string):boolean;
var pos:integer;
    switchMode:boolean;
    i,l:integer;
    s,sw:string;
begin
pos := 1;
switchMode := true;
l:=ParamCount;
 for i := 1 to 2 do begin
    if i = 1 then s := inF;
    if i = 2 then s := outF;
    if length(s) = 0 then begin
       result:=false;
       exit;
       end;
    if switchMode then begin
       if comparestr(s,'--')= 0 then begin
          switchMode := false;
          continue;
          end;
       if s[1]='-' then begin
          sw := AnsiLowerCase(GetStr(s,2));
          if length(sw) = 0 then begin
             result:=false;
             exit;
             end;
          try
             if not ParseSwitch(sw) then begin
                result:=false;
                exit;
                end;
             except on e:EConvertError do begin
                    result:=false;
                    exit;
                    end;
             end;
          continue;
          end;
       end;
       if pos = 3 then begin
          if comparetext(s,'e')=0 then
             Command := kEncode
          else if comparetext(s,'d')=0 then
             Command := kDecode
          else if comparetext(s,'b')=0 then
             Command := kBenchmark
          else begin
               result:=false;
               exit;
               end;
          end
        else if pos = 1 then begin
             if Command = kBenchmark then begin
                try
                   NumBenchmarkPasses := strtoint(s);
                   if NumBenchmarkPasses < 1 then begin
                      result:=false;
                      exit;
                      end;
                   except on e:EConvertError do begin
                          result:=false;
                          exit;
                          end;
                   end;
                end
             else InFile := s;
             end
        else if pos = 2 then
             OutFile := s
        else begin
             result:=false;
             exit;
             end;
    inc(pos);
    continue;
    end;
result:=true;
exit;
end;

procedure TLZMAAlone.PrintHelp;
begin

end;

function TLZMAAlone.Main(inF,outF,Cmd:string):boolean;
var params:TCommandLine;
    dictionary:integer;
   // lzmaBench:tlzmabench;
    inStream:TBufferedFS;
    outStream:TBufferedFS;
    eos:boolean;
    encoder:TLZMAEncoder;
    filesize:int64;
    i:integer;
    properties:array[0..4] of byte;
    decoder:TLZMADecoder;
    outSize:int64;
    v:byte;
const propertiessize=5;
begin
  result:=true;
  Params:=TCommandLine.Create;
  try
  if (not params.Parse(inF,outF)) then Result:=false;

  inStream:=TBufferedFS.Create(params.InFile,fmOpenRead or fmsharedenynone);
  outStream:=TBufferedFS.Create(params.OutFile,fmcreate);
  eos := false;
  if params.Eos then
  eos := true;
  if LowerCase(Cmd) = 'e' then begin
    encoder:=TLZMAEncoder.Create;
           if not encoder.SetAlgorithm(params.Algorithm) then raise Exception.Create('Incorrect compression mode');
           if not encoder.SetDictionarySize(params.DictionarySize) then raise Exception.Create('Incorrect dictionary size');
           if not encoder.SeNumFastBytes(params.Fb) then raise Exception.Create('Incorrect -fb value');
           if not encoder.SetMatchFinder(params.MatchFinder) then raise Exception.Create('Incorrect -mf value');
           if not encoder.SetLcLpPb(params.Lc, params.Lp, params.Pb) then raise Exception.Create('Incorrect -lc or -lp or -pb value');
           encoder.SetEndMarkerMode(eos);
           encoder.WriteCoderProperties(outStream);
        if eos then fileSize := -1
           else fileSize := inStream.Size;
        for i := 0 to 7 do
            WriteByte(outStream,(fileSize shr (8 * i)) and $FF);
        encoder.Code(inStream, outStream, -1, -1);
        encoder.free;
        end
        else
        if Cmd = 'd' then
        begin
             if inStream.read(properties, propertiesSize) <> propertiesSize then
               // raise Exception.Create('input .lzma file is too short');
                begin
               result:=False;
               Exit;
               end;
             decoder := TLZMADecoder.Create;
             try
              if not decoder.SetDecoderProperties(properties) then
               // raise Exception.Create('Incorrect stream properties');
               begin
               result:=False;
               Exit;
               end;
             outSize := 0;
             for i := 0 to 7 do begin
                 v := {shortint}(ReadByte(inStream));
                 if v < 0 then
                   // raise Exception.Create('Can''t read stream size');
                   begin
                      result:=False;
                      Exit;
                   end;
                 outSize := outSize or v shl (8 * i);
                 end;
             if not decoder.Code(inStream, outStream, outSize) then
             //   raise Exception.Create('Error in data stream');
             begin
             result:=False;
             Exit;
             end;
             finally
             decoder.Free;
             end;
        end;
  finally
   outStream.Free;
   inStream.Free;
   Params.Free;
  end;


end;

end.
